;(function($) {

		var file_uploader = function( elem ) {
		
			var options = $.extend({}, $.fn.file_uploader.defaults);
			
			// common variables
			var thisClass = this,
				$elem,
				iBytesUploaded,
				iBytesTotal,
				iPreviousBytesLoaded,
				iMaxFilesize,
				oTimer,
				sResultFileSize;

			$elem = $(elem);
			
			//Create the Form
			$form = $('<form></form>');
			$form.attr('action' , $elem.data('url'));
			$form.attr('method' , 'post');
			$form.attr('enctype' , 'multipart/form-data');
			$form.addClass('well');

			//create message container
			$messages = $('<div class="upload_messages alert" style="display:none;"></div>');

			//Add messages container to the form
			$form.append($messages);		

			// create_upload_buttom
			$div = $('<div></div>');
			$span = $('<span class="btn btn-default btn-file"></span>');
			$span.append( $('<span class="fileupload-new"><i class="icon-file"></i> Enviar Arquivo</span>') );

			//create input and add handler
			$input = $('<input type="file" name="user_file" />');
			$input.on('change', function () { thisClass.upload_handler(this); } );

			$span.append($input);

			$div.append($span);
			$form.append($div);
			$elem.append($form);

			this.upload_handler = function(object)
			{
				thisClass.hide_messages();
				var object_file = $(object).prop('files')[0];
				var file_ext = object_file.name.substr((~-object_file.name.lastIndexOf(".") >>> 0) + 2);
				var allowed_ext = $elem.data('ext').split(","); 

				if ( allowed_ext.indexOf(file_ext)  == -1) 
				{
					thisClass.message(options.messages.error , 'error');
					return;
				};

				// little test for filesize
				if (object_file.size > iMaxFilesize) 
				{
					thisClass.message(options.messages.warnsize , '');
					return;
				}

				// prepare HTML5 FileReader
				var oReader = new FileReader();
				oReader.onload = function(e){};
				oReader.onloadend = function(e){ thisClass.upload_start(object_file)};

				// read selected file as DataURL
				oReader.readAsDataURL(object_file);  				
			}

			this.create_progress = function()
			{
				$progress 	= $('<div class="progress progress-striped active" style="margin-top:10px;"></div>');
				$bar 		= $('<div class="bar" style="width:0%;" ></div>');
				$span 		= $('<span class="width" style="padding:2px;">0%</span>');
				$progress.append($bar).append($span);
				$speed 		= $('<p class="text-info speed" style="padding:2px;"></p>');
				
				$elem.find('form').append($progress).append($speed);   
			}

			this.update_progress = function(percent)
			{
				var progress = percent+'%';
				$elem.find('.progress .bar').width(progress);   
				$elem.find('.progress .width').html(progress);   
			}

			this.destroy_progress = function(e)
			{
				$elem.find('.progress').remove();   
				$elem.find('.speed').remove();   
			}

			this.upload_start = function(object_file)
			{	
				$elem.find('.btn-file').hide();
				// cleanup all temp states
			    iPreviousBytesLoaded = 0;
			    thisClass.hide_messages();
			    thisClass.create_progress();

			    // get form data for POSTing

			    var virtual_form = new FormData(); 
			    virtual_form.append('files', object_file);

			    // var action = $form.attr('action');

			    // create XMLHttpRequest object, adding few event listeners, and POSTing our data
			    var oXHR = new XMLHttpRequest();        
			    oXHR.upload.addEventListener('progress', thisClass.upload_progress, false);
			    oXHR.addEventListener('load', thisClass.upload_finish, false);
			    oXHR.addEventListener('error', thisClass.upload_error, false);
			    oXHR.addEventListener('abort', thisClass.upload_abort, false);
			    oXHR.open('POST', $form.attr('action'));
			    oXHR.send(virtual_form);

			    // set inner timer
			    oTimer = setInterval(thisClass.do_inner_updates, 300);
			}

			//Common functions
			this.message = function(message, type) 
			{ 
				$close = '';
				$elem.find('.upload_messages').attr('class','upload_messages alert alert-'+type).html($close).append(message).show();
			}

			this.hide_messages = function()
			{	
				$elem.find('.upload_messages').hide();
			} 

			this.seconds_to_time = function(secs)
			{
			    var hr = Math.floor(secs / 3600);
			    var min = Math.floor((secs - (hr * 3600))/60);
			    var sec = Math.floor(secs - (hr * 3600) -  (min * 60));

			    if (hr < 10) {hr = "0" + hr; }
			    if (min < 10) {min = "0" + min;}
			    if (sec < 10) {sec = "0" + sec;}
			    if (hr) {hr = "00";}
			    return hr + ':' + min + ':' + sec;
			}

			this.bytes_to_size = function(bytes)
			{
			    var sizes = ['Bytes', 'KB', 'MB'];
			    if (bytes == 0) return 'n/a';
			    var i = parseInt(Math.floor(Math.log(bytes) / Math.log(1024)));
			    return (bytes / Math.pow(1024, i)).toFixed(1) + ' ' + sizes[i];
			}

			this.upload_error = function(e)
			{	
				thisClass.message(options.messages.error2 , 'error');
			    clearInterval(oTimer);
			}  

			this.upload_abort = function(e)
			{	
				$elem.find('.btn-file').show();
				thisClass.message(options.messages.abort , 'error');
			    clearInterval(oTimer);
			}

			this.upload_progress = function(e) // upload process in progress
			{	
			    if (e.lengthComputable) 
			    {
			        iBytesUploaded = e.loaded;
			        iBytesTotal = e.total;
			        var iPercentComplete = Math.round(e.loaded * 100 / e.total);
			        var iBytesTransfered = thisClass.bytes_to_size(iBytesUploaded);

			        thisClass.update_progress(iPercentComplete.toString());

			        if (iPercentComplete == 100) 
			        {
			    		thisClass.message(options.messages.processing , 'info');
			    		thisClass.destroy_progress();
			        }
			    } else {
			    	thisClass.message(options.messages.compute , 'error');
			    }			    
			}

			this.do_inner_updates = function(e) // we will use this function to display upload speed
			{ 
			    var iCB = iBytesUploaded;
			    var iDiff = iCB - iPreviousBytesLoaded;

			    // if nothing new loaded - exit
			    if (iDiff == 0)
			    {
			        return;			    	
			    }

			    iPreviousBytesLoaded = iCB;
			    iDiff = iDiff * 2;
			    var iBytesRem = iBytesTotal - iPreviousBytesLoaded;
			    var secondsRemaining = iBytesRem / iDiff;

			    // update speed info
			    var iSpeed = iDiff.toString() + 'B/s';
			    if (iDiff > 1024 * 1024) {
			        iSpeed = (Math.round(iDiff * 100/(1024*1024))/100).toString() + 'MB/s';
			    } else if (iDiff > 1024) {
			        iSpeed =  (Math.round(iDiff * 100/1024)/100).toString() + 'KB/s';
			    }
			    $elem.find('.speed').html( iSpeed + ' | ' + thisClass.seconds_to_time(secondsRemaining));
			}

			this.upload_finish = function(e)
			{	
				thisClass.destroy_progress();			    
			    var response = e.target.responseText;

			    if (response == 'error') 
			    {
			    	thisClass.upload_error();
			    } 
			    else
			    {
				    thisClass.check_file(response);					
				    clearInterval(oTimer);
			    }
			}

			this.check_file = function(filename)
			{	
				thisClass.message(options.messages.loading , 'info');
			    var url = $form.attr('action');
			    url = url.replace('upload', 'upload/check_file')+'/' + filename;
			    $.get(url, function(data) 
			    {

			    	thisClass.message(options.messages.success , 'success');
			    	if (data == 'success') 
			    	{
			    		$('#option_upload').val('uploads/pedidos/'+filename);
			    	}
			    });
			}		

		}; //plugin class ends

		jQuery.fn.file_uploader = function( ) {
			
			return this.each(function()
			{
				var element = $(this);

	            // Return early if this element already has a plugin instance
	            if (element.data('file-uploader')) { return };

	            var pup = new file_uploader(this);

	            // Store plugin object in this element's data
	            element.data('file-uploader', pup);
	            
			});
		};
		
		$.fn.file_uploader.defaults = {
			iBytesUploaded : 0,
			iBytesTotal : 0,
			iPreviousBytesLoaded : 0,
			iMaxFilesize : 50*1048576,
			oTimer : 0,
			messages : {
				error : 'O tipo de arquivo selecionado não é permitido.',
			    error2 : 'Ocorreu um erro ao enviar seu arquivo.',
			    abort : 'O envio foi cancelado pelo navegador ou a sua conexão caiu.',
			    compute : 'Não foi possível calcular o progresso.',
			    success : 'Seu arquivo foi enviado com sucesso.',
			    processing : 'Por favor aguarde enquanto verificamos seu arquivo.',
			    loading : '<img src="http://loja.tmxprodutos.com.br/assets/img/storing_animation.gif" style="width:20px; height:20px;" />  Por Favor aguarde enquanto analizamos seu arquivo...',
			    warnsize : 'Seu arquivo é grande demais. Não podemos aceitá-lo. Por favor selecione um arquivo menor.'
			},
			sResultFileSize : ''
		};

	})(jQuery);
